@@ -6,12 +6,10 @@ use keeshond_datapack::{DataId, DataStore, DataHandle};
 use keeshond_treats::visual::{Costume, Animation, AnimationCel, SliceHandle};
 
 use crate::document::{Document, DocumentState, ActionHistory};
-use crate::editorgui::{ResourceType, Packages, EditorConfig};
+use crate::editorgui::{ResourceType, Packages, EditorConfig, ProjectConfig};
 
 use std::any::Any;
 
-use imgui::{im_str, Ui};
-
 const CEL_BUTTON_SIZE : f32 = 96.0;
 
 enum CostumeAction
@@ -120,7 +118,7 @@ impl CostumeDocument
         draw_list.add_line([x, y - cross_size], [x, y + cross_size], imgui::ImColor32::from(cross_color)).thickness(1.0).build();
     }
 
-    fn animation_view(&mut self, ui : &mut Ui, costume : &mut Costume)
+    fn animation_view(&mut self, ui : &imgui::Ui, costume : &mut Costume)
     {
         let main_style = ui.clone_style();
 
@@ -132,31 +130,27 @@ impl CostumeDocument
             {
                 let mut update_cel = false;
 
-                ui.text(&im_str!("Editing Cel {}", self.current_cel));
+                ui.text(format!("Editing Cel {}", self.current_cel));
                 ui.spacing();
 
-                let mut package_name = imgui::ImString::with_capacity(4096);
-                let mut sheet_name = imgui::ImString::with_capacity(4096);
-                let mut slice_name = imgui::ImString::with_capacity(4096);
-
-                package_name.push_str(cel.sheet.package());
-                sheet_name.push_str(cel.sheet.path());
-                slice_name.push_str(&cel.slice.slice());
+                let mut package_name = cel.sheet.package().clone();
+                let mut sheet_name = cel.sheet.path().clone();
+                let mut slice_name = cel.slice.slice().clone();
 
                 let mut delay = cel.delay as f32;
                 let mut offset = [cel.x, cel.y];
 
                 let style_var = ui.push_style_var(imgui::StyleVar::ItemSpacing([-1.0, -1.0]));
 
-                update_cel |= ui.input_text(im_str!("Package"), &mut package_name).enter_returns_true(true).build();
-                update_cel |= ui.input_text(im_str!("Sheet"), &mut sheet_name).enter_returns_true(true).build();
+                update_cel |= ui.input_text("Package", &mut package_name).enter_returns_true(true).build();
+                update_cel |= ui.input_text("Sheet", &mut sheet_name).enter_returns_true(true).build();
 
-                style_var.pop(ui);
+                style_var.pop();
 
-                update_cel |= ui.input_text(im_str!("Slice"), &mut slice_name).enter_returns_true(true).build();
+                update_cel |= ui.input_text("Slice", &mut slice_name).enter_returns_true(true).build();
 
-                update_cel |= imgui::Drag::new(im_str!("Delay")).display_format(im_str!("%.4f")).build(ui, &mut delay);
-                update_cel |= imgui::Drag::new(im_str!("Offset")).display_format(im_str!("%.4f")).build_array(ui, &mut offset);
+                update_cel |= imgui::Drag::new("Delay").display_format("%.4f").build(ui, &mut delay);
+                update_cel |= imgui::Drag::new("Offset").display_format("%.4f").build_array(ui, &mut offset);
 
                 ui.spacing();
 
@@ -164,8 +158,8 @@ impl CostumeDocument
                 {
                     let new_cel = AnimationCel
                     {
-                        sheet : DataHandle::new(package_name.to_str(), sheet_name.to_str()),
-                        slice : SliceHandle::new(slice_name.to_str()),
+                        sheet : DataHandle::new(&package_name, &sheet_name),
+                        slice : SliceHandle::new(&slice_name),
                         delay : delay as f64,
                         x : offset[0],
                         y : offset[1]
@@ -192,32 +186,26 @@ impl CostumeDocument
                 }
             }
 
-            ui.text(&im_str!("Editing Animation {}", self.current_animation));
+            ui.text(format!("Editing Animation {}", self.current_animation));
             ui.spacing();
 
             let mut update_animation = false;
-
-            let mut animation_name = imgui::ImString::with_capacity(4096);
-
-            if let Some(name) = costume.animation_name(self.current_animation)
-            {
-                animation_name.push_str(&name);
-            }
+            let mut animation_name = costume.animation_name(self.current_animation).unwrap_or_default();
 
             if self.focus_animation_name_input
             {
-                ui.set_keyboard_focus_here(imgui::FocusedWidget::Offset(0));
+                ui.set_keyboard_focus_here_with_offset(imgui::FocusedWidget::Offset(0));
                 self.focus_animation_name_input = false;
             }
 
-            if ui.input_text(im_str!("Name"), &mut animation_name).enter_returns_true(true).build()
+            if ui.input_text("Name", &mut animation_name).enter_returns_true(true).build()
             {
                 self.queue_action(Box::new(CostumeAction::RenameAnimation(self.current_animation, animation_name.to_string())));
             }
 
             let mut looping = animation.looping();
 
-            update_animation |= ui.checkbox(im_str!("Looping"), &mut looping);
+            update_animation |= ui.checkbox("Looping", &mut looping);
 
             if update_animation
             {
@@ -230,22 +218,22 @@ impl CostumeDocument
 
         ui.spacing();
 
-        if ui.button(im_str!("Add"), [0.0, 0.0])
+        if ui.button("Add")
         {
             self.queue_action(Box::new(CostumeAction::AddAnimation(self.current_animation, Animation::from_cels(Vec::new(), false))));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Up"), [0.0, 0.0]) && self.current_animation > 0
+        if ui.button("Up") && self.current_animation > 0
         {
             self.queue_action(Box::new(CostumeAction::MoveAnimation(self.current_animation, self.current_animation - 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Down"), [0.0, 0.0])
+        if ui.button("Down")
         {
             self.queue_action(Box::new(CostumeAction::MoveAnimation(self.current_animation, self.current_animation + 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Remove"), [0.0, 0.0])
+        if ui.button("Remove")
         {
             self.queue_action(Box::new(CostumeAction::RemoveAnimation(self.current_animation)));
         }
@@ -263,9 +251,11 @@ impl CostumeDocument
 
                 if let Some(name) = costume.animation_name(i)
                 {
-                    animation_name = im_str!("{}", name);
-                } else {
-                    animation_name = im_str!("<animation {}>", i);
+                    animation_name = name;
+                }
+                else
+                {
+                    animation_name = format!("<animation {}>", i);
                 }
 
                 if imgui::Selectable::new(&animation_name).selected(self.current_animation == i).build(ui)
@@ -283,11 +273,11 @@ impl CostumeDocument
             }
         });
 
-        colors_animation_list.pop(ui);
-        style_animation_list.pop(ui);
+        colors_animation_list.pop();
+        style_animation_list.pop();
     }
 
-    fn filmstrip(&mut self, packages : &mut Packages, ui : &mut Ui, mut sheet_store : &mut DataStore<Sheet>, animation : &Animation)
+    fn filmstrip(&mut self, packages : &mut Packages, ui : &imgui::Ui, mut sheet_store : &mut DataStore<Sheet>, animation : &Animation)
     {
         // FIXME ImGui currently has a bug where adding a row of images extends the vertical scroll when it shouldn't.
         //  For now, turn off the vertical scrolling controls and force scroll Y to 0.
@@ -307,18 +297,18 @@ impl CostumeDocument
                     main_button_color = transparent_color;
                 }
 
-                let cel_button_color = ui.push_style_colors(&[
-                    (imgui::StyleColor::Border, crate::util::color_to_border_color(transparent_color)),
-                    (imgui::StyleColor::Button, main_button_color)]);
+                let _cel_button_colors = [
+                    ui.push_style_color(imgui::StyleColor::Border, crate::util::color_to_border_color(transparent_color)),
+                    ui.push_style_color(imgui::StyleColor::Button, main_button_color)];
 
-                if ui.button(&im_str!("###cel{}", i), [CEL_BUTTON_SIZE, CEL_BUTTON_SIZE])
+                if ui.button_with_size(format!("###cel{}", i), [CEL_BUTTON_SIZE, CEL_BUTTON_SIZE])
                 {
                     self.current_cel = i;
                 }
 
                 let (button_min, button_max) = (ui.item_rect_min(), ui.item_rect_max());
 
-                ui.same_line(0.0);
+                ui.same_line();
                 let new_cursor_pos = ui.cursor_pos();
 
                 if self.current_cel == i && self.scroll_cel_list
@@ -342,23 +332,21 @@ impl CostumeDocument
                 }
 
                 ui.set_cursor_pos(new_cursor_pos);
-
-                cel_button_color.pop(ui);
             }
         }
     }
 
-    fn filmstrip_buttons(&mut self, packages : &mut Packages, ui : &mut Ui, sheet_store : &mut DataStore<Sheet>, animation : &Animation)
+    fn filmstrip_buttons(&mut self, packages : &mut Packages, ui : &imgui::Ui, sheet_store : &mut DataStore<Sheet>, animation : &Animation)
     {
-        ui.popup(im_str!("Add Cel Menu"), ||
+        ui.popup("Add Cel Menu", ||
         {
-            if imgui::Selectable::new(im_str!("Empty Cel")).close_popups(false).build(ui)
+            if imgui::Selectable::new("Empty Cel").close_popups(false).build(ui)
             {
                 self.queue_action(Box::new(CostumeAction::AddCel(self.current_animation, self.current_cel, AnimationCel::new())));
             }
             if let Some(old_cel) = animation.cel(self.current_cel)
             {
-                if imgui::Selectable::new(im_str!("Clone Cel")).close_popups(false).build(ui)
+                if imgui::Selectable::new("Clone Cel").close_popups(false).build(ui)
                 {
                     self.queue_action(Box::new(CostumeAction::AddCel(self.current_animation, self.current_cel, old_cel.clone())));
                 }
@@ -370,7 +358,7 @@ impl CostumeDocument
 
             for package in packages.package_iter()
             {
-                ui.menu(&im_str!("{}", package), true, ||
+                ui.menu(&package, ||
                 {
                     if !packages.package_loaded(ResourceType::Sheet, package)
                     {
@@ -379,7 +367,7 @@ impl CostumeDocument
 
                     for sheet_name in packages.package_contents(ResourceType::Sheet, package)
                     {
-                        ui.menu(&im_str!("{}", sheet_name), true, ||
+                        ui.menu(&sheet_name, ||
                         {
                             let sheet_id = sheet_store.get_id(package, sheet_name).unwrap_or(0);
 
@@ -389,7 +377,7 @@ impl CostumeDocument
                                 {
                                     if let Some(slice_name) = sheet.slice_name(slice_id)
                                     {
-                                        if imgui::Selectable::new(&im_str!("{}", slice_name)).close_popups(false).build(ui)
+                                        if imgui::Selectable::new(&slice_name).close_popups(false).build(ui)
                                         {
                                             let mut cel = AnimationCel::new();
 
@@ -412,28 +400,28 @@ impl CostumeDocument
             }
         });
 
-        if ui.button(im_str!("Add###Add Cel"), [0.0, 0.0])
+        if ui.button("Add###Add Cel")
         {
-            ui.open_popup(im_str!("Add Cel Menu"));
+            ui.open_popup("Add Cel Menu");
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Left"), [0.0, 0.0]) && self.current_cel > 0
+        if ui.button("Left") && self.current_cel > 0
         {
             self.queue_action(Box::new(CostumeAction::MoveCel(self.current_animation, self.current_cel, self.current_cel - 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Right"), [0.0, 0.0])
+        if ui.button("Right")
         {
             self.queue_action(Box::new(CostumeAction::MoveCel(self.current_animation, self.current_cel, self.current_cel + 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Remove###Remove Cel"), [0.0, 0.0])
+        if ui.button("Remove###Remove Cel")
         {
             self.queue_action(Box::new(CostumeAction::RemoveCel(self.current_animation, self.current_cel)));
         }
 
         ui.same_line_with_spacing(0.0, 40.0);
-        ui.checkbox(im_str!("Play Animation"), &mut self.play_animation);
+        ui.checkbox("Play Animation", &mut self.play_animation);
     }
 }
 
@@ -556,7 +544,8 @@ impl Document for CostumeDocument
         false
     }
 
-    fn do_ui(&mut self, packages : &mut Packages, _config : &mut EditorConfig, game: &mut GameControl, ui : &mut imgui::Ui) -> DocumentState
+    fn do_ui(&mut self, packages : &mut Packages, _config : &mut EditorConfig,
+             _project : &mut ProjectConfig, game : &mut GameControl, ui : &imgui::Ui) -> DocumentState
     {
         self.tics = self.tics.wrapping_add(1);
 
@@ -564,23 +553,23 @@ impl Document for CostumeDocument
 
         ui.menu_bar(||
         {
-            ui.menu(im_str!("View"), true, ||
+            ui.menu("View", ||
             {
-                if imgui::MenuItem::new(im_str!("Show Origin")).selected(self.show_origin).build(ui)
+                if imgui::MenuItem::new("Show Origin").selected(self.show_origin).build(ui)
                 {
                     self.show_origin = !self.show_origin;
                 }
 
-                ui.menu(im_str!("Background Color"), true, ||
+                ui.menu("Background Color", ||
                 {
-                    imgui::ColorPicker::new(im_str!("##background_color_picker"), imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
+                    imgui::ColorPicker::new("##background_color_picker", imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
                 });
             });
-            ui.menu(im_str!("Zoom"), true, ||
+            ui.menu("Zoom", ||
             {
                 for level in &[0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0]
                 {
-                    if imgui::MenuItem::new(&im_str!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
+                    if imgui::MenuItem::new(format!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
                     {
                         self.zoom_level = *level;
                     }
@@ -595,7 +584,7 @@ impl Document for CostumeDocument
 
         if let Some(costume) = costume_store.get_mut(self.data_id)
         {
-            ui.columns(2, im_str!("##costume_columns"), false);
+            ui.columns(2, "##costume_columns", false);
 
             if let Some(costume_controls) = imgui::ChildWindow::new("costume_controls").size([240.0, 0.0])
                 .draw_background(true).border(false).horizontal_scrollbar(true).begin(ui)
@@ -604,10 +593,10 @@ impl Document for CostumeDocument
                 {
                     self.animation_view(ui, costume);
 
-                    animation_view.end(ui);
+                    animation_view.end();
                 }
 
-                costume_controls.end(ui);
+                costume_controls.end();
             }
 
             ui.set_current_column_width(ui.item_rect_size()[0] + 10.0);
@@ -617,7 +606,8 @@ impl Document for CostumeDocument
             {
                 let main_style = ui.clone_style();
 
-                let style_filmstrip = ui.push_style_vars(&[imgui::StyleVar::ItemSpacing([-1.0, 2.0]), imgui::StyleVar::WindowPadding([4.0, 4.0])]);
+                let style_filmstrip_spacing = ui.push_style_var(imgui::StyleVar::ItemSpacing([-1.0, 2.0]));
+                let style_filmstrip_padding = ui.push_style_var(imgui::StyleVar::WindowPadding([4.0, 4.0]));
                 let colors_filmstrip = ui.push_style_color(imgui::StyleColor::ChildBg, main_style.colors[imgui::StyleColor::FrameBg as usize]);
 
                 self.current_cel = self.current_cel.min((animation.cel_count()).max(1) - 1);
@@ -627,11 +617,12 @@ impl Document for CostumeDocument
                 {
                     self.filmstrip(packages, ui, &mut sheet_store, animation);
 
-                    filmstrip.end(ui);
+                    filmstrip.end();
                 }
 
-                colors_filmstrip.pop(ui);
-                style_filmstrip.pop(ui);
+                colors_filmstrip.pop();
+                style_filmstrip_padding.pop();
+                style_filmstrip_spacing.pop();
 
                 self.filmstrip_buttons(packages, ui, &mut sheet_store, animation);
 
@@ -668,12 +659,12 @@ impl Document for CostumeDocument
                     }
                     else
                     {
-                        ui.text(im_str!("No cel selected!"));
+                        ui.text("No cel selected!");
                     }
                 });
 
-                style_color.pop(ui);
-                style_var.pop(ui);
+                style_color.pop();
+                style_var.pop();
             }
         }
 
