@@ -1,20 +1,16 @@
 use crate::document::{Document, DocumentState, ActionHistory};
-use crate::editorgui::{Packages, EditorConfig};
+use crate::editorgui::{Packages, EditorConfig, ProjectConfig};
+use crate::import::sheet::TilesetGenInfo;
 
 use keeshond::gameloop::GameControl;
 use keeshond::renderer::{Sheet, SpriteSlice};
 use keeshond_datapack::DataId;
 
 use std::any::Any;
-use std::ops::{RangeInclusive, Deref, DerefMut};
 
-use image::{RgbaImage, GrayImage, ImageBuffer, GenericImage, GenericImageView};
-use image::math::Rect;
-use imgui::{im_str, TabBarFlags, ImStr, Ui};
+use image::{RgbaImage, GrayImage};
 
 const MAX_DRAW_POINTS : usize = 32768;
-const MAX_TEXTURE_TILE_DIMENSIONS : u32 = 2048;
-const MAX_TEXTURE_TILE_DIMENSIONS_MANUAL : u32 = 128;
 
 enum SheetAction
 {
@@ -30,29 +26,6 @@ enum SheetAction
     RemoveSlices(usize, usize)
 }
 
-struct TransparentPixel
-{
-    x : u32, y : u32, set : bool
-}
-
-fn automatic_or_number_printf(number : u32) -> &'static ImStr
-{
-    if number == 0
-    {
-        return im_str!("Automatic");
-    }
-
-    im_str!("%d")
-}
-
-struct TilesetSliceInfo
-{
-    rows : u32,
-    columns : u32,
-    tile_walk_x : u32,
-    tile_walk_y : u32
-}
-
 pub struct SheetDocument
 {
     action_history : ActionHistory,
@@ -69,14 +42,7 @@ pub struct SheetDocument
     mouse_drawing_texture : bool,
     draw_points : Vec<(f32, f32)>,
     dragging : bool,
-    tilegen_w : u32,
-    tilegen_h : u32,
-    tilegen_x : u32,
-    tilegen_y : u32,
-    tilegen_space_x : u32,
-    tilegen_space_y : u32,
-    tilegen_rows : u32,
-    tilegen_columns : u32,
+    tilegen : TilesetGenInfo,
     tilegen_preview : bool,
     on_tilegen : bool
 }
@@ -158,89 +124,8 @@ impl SheetDocument
     {
         let mut pixels = sheet.data();
         let mut texture = RgbaImage::from_raw(sheet.width(), sheet.height(), pixels).expect("Raw data to image failed");
-        let mut trans_pixels = Vec::new();
 
-        for y in 0..texture.height()
-        {
-            for x in 0..texture.width()
-            {
-                let pixel = texture.get_pixel_mut(x, y);
-
-                if pixel[3] == 0
-                {
-                    trans_pixels.push(TransparentPixel
-                    {
-                        x, y, set : false
-                    })
-                }
-            }
-        }
-
-        let mut colors = Vec::new();
-
-        while trans_pixels.len() > 0
-        {
-            let mut processed = false;
-
-            for entry in &mut trans_pixels
-            {
-                colors.clear();
-                let (x, y) = (entry.x, entry.y);
-
-                for (offset_x, offset_y) in
-                    &[
-                        (-1, -1), (0, -1), (1, -1),
-                        (-1, 0), (1, 0),
-                        (-1, 1), (0, 1), (1, 1)
-                    ]
-                {
-                    let check_x = (x as i32 + offset_x) as u32 % texture.width();
-                    let check_y = (y as i32 + offset_y) as u32 % texture.height();
-
-                    let other_pixel = texture.get_pixel(check_x, check_y);
-
-                    if other_pixel[3] > 0
-                    {
-                        colors.push((other_pixel[0], other_pixel[1], other_pixel[2]));
-                    }
-                }
-
-                if colors.len() > 0
-                {
-                    let mut red = 0.0;
-                    let mut green = 0.0;
-                    let mut blue = 0.0;
-
-                    for (other_r, other_g, other_b) in &colors
-                    {
-                        red += *other_r as f32;
-                        green += *other_g as f32;
-                        blue += *other_b as f32;
-                    }
-
-                    red /= colors.len() as f32;
-                    green /= colors.len() as f32;
-                    blue /= colors.len() as f32;
-
-                    let pixel = texture.get_pixel_mut(x, y);
-
-                    pixel[0] = red as u8;
-                    pixel[1] = green as u8;
-                    pixel[2] = blue as u8;
-
-                    processed = true;
-                    entry.set = true;
-                }
-            }
-
-            trans_pixels.retain(|entry| entry.set == false);
-
-            // Don't get stuck if there are no opaque pixels
-            if !processed
-            {
-                break;
-            }
-        }
+        crate::util::defringe_image(&mut texture, true);
 
         pixels = texture.into_raw();
 
@@ -420,41 +305,14 @@ impl SheetDocument
             });
     }
 
-    fn tileset_slice_info(&self, width : u32, height : u32) -> TilesetSliceInfo
-    {
-        let tile_walk_x = self.tilegen_w + self.tilegen_space_x;
-        let tile_walk_y = self.tilegen_h + self.tilegen_space_y;
-        let max_columns = (width - self.tilegen_x.min(width) + self.tilegen_space_x) / tile_walk_x;
-        let max_rows = (height - self.tilegen_y.min(height) + self.tilegen_space_y) / tile_walk_y;
-        let mut columns = self.tilegen_columns.min(max_columns);
-        let mut rows = self.tilegen_rows.min(max_rows);
-
-        if columns == 0
-        {
-            columns = max_columns;
-        }
-        if rows == 0
-        {
-            rows = max_rows;
-        }
-
-        TilesetSliceInfo
-        {
-            rows,
-            columns,
-            tile_walk_x,
-            tile_walk_y
-        }
-    }
-
     fn draw_tileset_preview(&self, pos: [f32; 2], draw_list : &imgui::DrawListMut, sheet : &Sheet)
     {
-        if self.tilegen_w as f32 * self.zoom_level < 8.0 || self.tilegen_h as f32 * self.zoom_level < 8.0
+        if self.tilegen.width as f32 * self.zoom_level < 8.0 || self.tilegen.height as f32 * self.zoom_level < 8.0
         {
             return;
         }
 
-        let tileset_info = self.tileset_slice_info(sheet.width(), sheet.height());
+        let tileset_info = crate::import::sheet::tileset_slice_info(sheet.width(), sheet.height(), &self.tilegen);
 
         let pulse = (((self.tics as f64 * 0.06).sin() + 1.0) / 2.0) as f32;
         let color = [pulse, pulse, pulse, 1.0];
@@ -464,34 +322,38 @@ impl SheetDocument
         {
             for x in 0..tileset_info.columns
             {
-                let x1 = pos[0] + (self.tilegen_x as f32 + (tileset_info.tile_walk_x * x) as f32) * self.zoom_level;
-                let y1 = pos[1] + (self.tilegen_y as f32 + (tileset_info.tile_walk_y * y) as f32) * self.zoom_level;
-                let x2 = x1 + (self.tilegen_w as f32) * self.zoom_level;
-                let y2 = y1 + (self.tilegen_h as f32) * self.zoom_level;
+                let x1 = pos[0] + (self.tilegen.x as f32 + (tileset_info.tile_walk_x * x) as f32) * self.zoom_level;
+                let y1 = pos[1] + (self.tilegen.y as f32 + (tileset_info.tile_walk_y * y) as f32) * self.zoom_level;
+                let x2 = x1 + (self.tilegen.width as f32) * self.zoom_level;
+                let y2 = y1 + (self.tilegen.height as f32) * self.zoom_level;
 
                 draw_list.add_rect([x1, y1], [x2, y2], imgui::ImColor32::from(color)).thickness(thickness).build();
             }
         }
     }
 
-    fn create_tileset_slices(&mut self, sheet : &Sheet, tileset_info : &TilesetSliceInfo)
+    fn create_tileset_from_sheet(&mut self, sheet : &Sheet)
     {
-        let mut slices = Vec::new();
+        let pixels = sheet.data();
+        let slices;
 
-        for y in 0..tileset_info.rows
+        if sheet.has_palette()
         {
-            for x in 0..tileset_info.columns
-            {
-                slices.push(SpriteSlice
-                {
-                    texture_x: self.tilegen_x as f32 + (tileset_info.tile_walk_x * x) as f32,
-                    texture_y: self.tilegen_y as f32 + (tileset_info.tile_walk_y * y) as f32,
-                    texture_w: self.tilegen_w as f32,
-                    texture_h: self.tilegen_h as f32,
-                    origin_x: 0.0,
-                    origin_y: 0.0
-                });
-            }
+            let texture = GrayImage::from_raw(sheet.width(), sheet.height(), pixels).expect("Raw data to image failed");
+            let (new_texture, new_slices) = crate::import::sheet::create_tileset_texture_and_slices(&texture, &mut self.tilegen);
+            let (width, height) = (new_texture.width(), new_texture.height());
+
+            self.queue_action(Box::new(SheetAction::SetPixelsWithSize(new_texture.into_raw(), width, height)));
+            slices = new_slices;
+        }
+        else
+        {
+            let texture = RgbaImage::from_raw(sheet.width(), sheet.height(), pixels).expect("Raw data to image failed");
+            let (new_texture, new_slices) = crate::import::sheet::create_tileset_texture_and_slices(&texture, &mut self.tilegen);
+            let (width, height) = (new_texture.width(), new_texture.height());
+
+            self.queue_action(Box::new(SheetAction::SetPixelsWithSize(new_texture.into_raw(), width, height)));
+            slices = new_slices;
         }
 
         if slices.len() > 0
@@ -501,100 +363,19 @@ impl SheetDocument
         }
     }
 
-    fn create_tileset_texture_internal<P, C>(&mut self, texture : &ImageBuffer<P, C>, new_texture : &mut ImageBuffer<P, C>, tileset_info : &TilesetSliceInfo)
-        where
-            P: image::Pixel + 'static,
-            P::Subpixel: 'static,
-            C: Deref<Target = [P::Subpixel]> + DerefMut,
-    {
-        let new_tile_width = self.tilegen_w + 2;
-        let new_tile_height = self.tilegen_h + 2;
-        let (width, _) = new_texture.dimensions();
-        let new_columns = width / new_tile_width;
-
-        for y in 0..tileset_info.rows
-        {
-            for x in 0..tileset_info.columns
-            {
-                let source_x = self.tilegen_x + (tileset_info.tile_walk_x * x);
-                let source_y = self.tilegen_y + (tileset_info.tile_walk_y * y);
-                let index = y * tileset_info.columns + x;
-                let dest_x = (index % new_columns) * new_tile_width;
-                let dest_y = (index / new_columns) * new_tile_height;
-
-                let mut tile_image = ImageBuffer::new(new_tile_width, new_tile_height);
-                let texture_view = texture.view(source_x, source_y, self.tilegen_w, self.tilegen_h);
-
-                // Main tile image
-                tile_image.copy_from(&texture_view, 1, 1).expect("Image copy failed");
-
-                // Tile image edge padding
-                tile_image.copy_within(Rect { x : 1, y : 1, width : 1, height : self.tilegen_h }, 0, 1);
-                tile_image.copy_within(Rect { x : self.tilegen_w, y : 1, width : 1, height : self.tilegen_h }, self.tilegen_w + 1, 1);
-                tile_image.copy_within(Rect { x : 0, y : 1, width : new_tile_width, height : 1 }, 0, 0);
-                tile_image.copy_within(Rect { x : 0, y : self.tilegen_h, width : new_tile_width, height : 1 }, 0, self.tilegen_h + 1);
-
-                new_texture.copy_from(&tile_image, dest_x, dest_y).expect("Image copy failed");
-            }
-        }
-    }
-
-    fn create_tileset_texture_and_slices(&mut self, sheet : &Sheet)
-    {
-        let mut tileset_info = self.tileset_slice_info(sheet.width(), sheet.height());
-
-        let width = (sheet.width() as f32).max(self.tilegen_w as f32 + 2.0).log2().ceil().exp2() as u32;
-        let total_tiles = tileset_info.columns * tileset_info.rows;
-        let new_columns = (width / (self.tilegen_w + 2)).min(MAX_TEXTURE_TILE_DIMENSIONS);
-        let new_rows = ((total_tiles as f32 / new_columns as f32).ceil() as u32).min(MAX_TEXTURE_TILE_DIMENSIONS);
-        let height = ((new_rows * (self.tilegen_h + 2)) as f32).log2().ceil().exp2() as u32;
-
-        let pixels = sheet.data();
-
-        if sheet.has_palette()
-        {
-            let texture = GrayImage::from_raw(sheet.width(), sheet.height(), pixels).expect("Raw data to image failed");
-            let mut new_texture = GrayImage::new(width, height);
-
-            self.create_tileset_texture_internal(&texture, &mut new_texture, &tileset_info);
-
-            self.queue_action(Box::new(SheetAction::SetPixelsWithSize(new_texture.into_raw(), width, height)));
-        }
-        else
-        {
-            let texture = RgbaImage::from_raw(sheet.width(), sheet.height(), pixels).expect("Raw data to image failed");
-            let mut new_texture = RgbaImage::new(width, height);
-
-            self.create_tileset_texture_internal(&texture, &mut new_texture, &tileset_info);
-
-            self.queue_action(Box::new(SheetAction::SetPixelsWithSize(new_texture.into_raw(), width, height)));
-        }
-
-        self.tilegen_x = 1;
-        self.tilegen_y = 1;
-        self.tilegen_space_x = 2;
-        self.tilegen_space_y = 2;
-        self.tilegen_columns = new_columns;
-        self.tilegen_rows = new_rows;
-
-        tileset_info = self.tileset_slice_info(width, height);
-
-        self.create_tileset_slices(sheet, &tileset_info);
-    }
-
-    fn sheet_tab(&mut self, ui : &mut Ui, sheet : &mut Sheet)
+    fn sheet_tab(&mut self, ui : &imgui::Ui, sheet : &mut Sheet)
     {
         let (mut force, mut force_on) = crate::util::option_bool_to_double_bool(sheet.texture_filtering());
         let mut update_filtering = false;
 
-        ui.text_wrapped(&im_str!("Size: {}x{}", sheet.width(), sheet.height()));
+        ui.text_wrapped(format!("Size: {}x{}", sheet.width(), sheet.height()));
 
-        update_filtering |= ui.checkbox(im_str!("Force texture filter setting"), &mut force);
+        update_filtering |= ui.checkbox("Force texture filter setting", &mut force);
 
         if force
         {
             ui.indent();
-            update_filtering |= ui.checkbox(im_str!("Enabled"), &mut force_on);
+            update_filtering |= ui.checkbox("Enabled", &mut force_on);
             ui.unindent();
         }
 
@@ -603,11 +384,11 @@ impl SheetDocument
             self.queue_action(Box::new(SheetAction::SetTextureFiltering(force, force_on)));
         }
 
-        if ui.button(im_str!("De-fringe"), [0.0, 0.0])
+        if ui.button("De-fringe")
         {
             if sheet.has_palette()
             {
-                ui.open_popup(im_str!("Can't Defringe"));
+                ui.open_popup("Can't Defringe");
             }
             else
             {
@@ -615,17 +396,17 @@ impl SheetDocument
             }
         }
 
-        ui.popup_modal(im_str!("Can't Defringe")).resizable(false).build(||
+        ui.popup_modal("Can't Defringe").resizable(false).build(ui, ||
         {
-            ui.text(im_str!("De-fringing requires averaging pixels,\nwhich can't be done for paletted images!"));
-            if ui.button(im_str!("Oh well!"), [0.0, 0.0])
+            ui.text("De-fringing requires averaging pixels,\nwhich can't be done for paletted images!");
+            if ui.button("Oh well!")
             {
                 ui.close_current_popup();
             }
         });
 
-        ui.same_line(0.0);
-        ui.text_disabled(im_str!("(?)"));
+        ui.same_line();
+        ui.text_disabled("(?)");
 
         if ui.is_item_hovered()
         {
@@ -634,29 +415,24 @@ impl SheetDocument
 
         ui.separator();
         ui.spacing();
-        ui.text(&im_str!("Editing Slice {}", self.current_slice));
+        ui.text(format!("Editing Slice {}", self.current_slice));
         ui.spacing();
 
-        let mut slice_name = imgui::ImString::with_capacity(4096);
+        let mut slice_name = sheet.slice_name(self.current_slice).unwrap_or_default();
 
         self.current_slice = self.current_slice.min(sheet.slice_count().max(1) - 1);
 
-        if let Some(name) = sheet.slice_name(self.current_slice)
-        {
-            slice_name.push_str(&name);
-        }
-
         if let Some(slice) = sheet.slice(self.current_slice)
         {
             let mut update_slice = false;
 
             if self.focus_slice_name_input
             {
-                ui.set_keyboard_focus_here(imgui::FocusedWidget::Offset(0));
+                ui.set_keyboard_focus_here_with_offset(imgui::FocusedWidget::Offset(0));
                 self.focus_slice_name_input = false;
             }
 
-            if ui.input_text(im_str!("Name"), &mut slice_name).enter_returns_true(true).build()
+            if ui.input_text("Name", &mut slice_name).enter_returns_true(true).build()
             {
                 self.queue_action(Box::new(SheetAction::RenameSlice(self.current_slice, slice_name.to_string())));
             }
@@ -667,12 +443,12 @@ impl SheetDocument
 
             let style_var = ui.push_style_var(imgui::StyleVar::ItemSpacing([-1.0, -1.0]));
 
-            update_slice |= imgui::Drag::new(im_str!("Pos")).display_format(im_str!("%.4f")).build_array(ui, &mut pos);
-            update_slice |= imgui::Drag::new(im_str!("Size")).display_format(im_str!("%.4f")).build_array(ui, &mut size);
+            update_slice |= imgui::Drag::new("Pos").display_format("%.4f").build_array(ui, &mut pos);
+            update_slice |= imgui::Drag::new("Size").display_format("%.4f").build_array(ui, &mut size);
 
-            style_var.pop(ui);
+            style_var.pop();
 
-            update_slice |= imgui::Drag::new(im_str!("Origin")).display_format(im_str!("%.4f")).build_array(ui, &mut origin);
+            update_slice |= imgui::Drag::new("Origin").display_format("%.4f").build_array(ui, &mut origin);
 
             if update_slice
             {
@@ -693,16 +469,16 @@ impl SheetDocument
         ui.separator();
         ui.spacing();
 
-        ui.checkbox(im_str!("Draw Tool"), &mut self.draw_tool_on);
+        ui.checkbox("Draw Tool", &mut self.draw_tool_on);
 
         if self.draw_tool_on
         {
             ui.indent();
-            ui.checkbox(im_str!("Add Padding"), &mut self.draw_tool_padding);
+            ui.checkbox("Add Padding", &mut self.draw_tool_padding);
             ui.unindent();
         }
 
-        if ui.button(im_str!("Add"), [0.0, 0.0])
+        if ui.button("Add")
         {
             self.queue_action(Box::new(SheetAction::AddSlice(self.current_slice, SpriteSlice
             {
@@ -715,17 +491,17 @@ impl SheetDocument
             })));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Up"), [0.0, 0.0]) && self.current_slice > 0
+        if ui.button("Up") && self.current_slice > 0
         {
             self.queue_action(Box::new(SheetAction::MoveSlice(self.current_slice, self.current_slice - 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Down"), [0.0, 0.0])
+        if ui.button("Down")
         {
             self.queue_action(Box::new(SheetAction::MoveSlice(self.current_slice, self.current_slice + 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Remove"), [0.0, 0.0])
+        if ui.button("Remove")
         {
             self.queue_action(Box::new(SheetAction::RemoveSlice(self.current_slice)));
         }
@@ -744,11 +520,11 @@ impl SheetDocument
 
                 if let Some(name) = sheet.slice_name(i)
                 {
-                    slice_name = im_str!("{}", name);
+                    slice_name = name;
                 }
                 else
                 {
-                    slice_name = im_str!("<slice {}>", i);
+                    slice_name = format!("<slice {}>", i);
                 }
 
                 if imgui::Selectable::new(&slice_name).selected(self.current_slice == i).build(ui)
@@ -764,35 +540,20 @@ impl SheetDocument
             }
         });
 
-        colors_slice_list.pop(ui);
-        style_slice_list.pop(ui);
+        colors_slice_list.pop();
+        style_slice_list.pop();
     }
 
-    fn tilegen_tab(&mut self, ui : &mut Ui, sheet : &mut Sheet)
+
+
+    fn tilegen_tab(&mut self, ui : &imgui::Ui, sheet : &mut Sheet)
     {
-        let mut size = [self.tilegen_w, self.tilegen_h];
-        let mut offset = [self.tilegen_x, self.tilegen_y];
-        let mut spacing = [self.tilegen_space_x, self.tilegen_space_y];
-
-        imgui::Drag::new(im_str!("Tile Size")).display_format(im_str!("%d px")).build_array(ui, &mut size);
-        imgui::Drag::new(im_str!("Start Offset")).display_format(im_str!("%d px")).build_array(ui, &mut offset);
-        imgui::Drag::new(im_str!("Tile Spacing")).display_format(im_str!("%d px")).build_array(ui, &mut spacing);
-        imgui::Drag::new(im_str!("Rows")).range(RangeInclusive::new(0, MAX_TEXTURE_TILE_DIMENSIONS_MANUAL))
-            .display_format(automatic_or_number_printf(self.tilegen_rows)).build(ui, &mut self.tilegen_rows);
-        imgui::Drag::new(im_str!("Columns")).range(RangeInclusive::new(0, MAX_TEXTURE_TILE_DIMENSIONS_MANUAL))
-            .display_format(automatic_or_number_printf(self.tilegen_columns)).build(ui, &mut self.tilegen_columns);
-
-        self.tilegen_w = size[0];
-        self.tilegen_h = size[1];
-        self.tilegen_x = offset[0];
-        self.tilegen_y = offset[1];
-        self.tilegen_space_x = spacing[0];
-        self.tilegen_space_y = spacing[1];
+        self.tilegen.tilegen_ui(ui);
 
         ui.spacing();
         ui.spacing();
 
-        ui.checkbox(im_str!("Preview"), &mut self.tilegen_preview);
+        ui.checkbox("Preview", &mut self.tilegen_preview);
 
         ui.spacing();
         ui.spacing();
@@ -800,26 +561,33 @@ impl SheetDocument
         ui.spacing();
         ui.spacing();
 
-        if ui.button(im_str!("Create Slices and Optimize Texture"), [0.0, 0.0])
+        if ui.button("Create Slices and Optimize Texture")
         {
-            self.create_tileset_texture_and_slices(sheet);
+            self.create_tileset_from_sheet(sheet);
         }
 
-        ui.text_wrapped(im_str!("Click this to get the sheet ready for use as a tileset. \
-                                Warning: This will modify the texture and replace any existing slices on this sheet!"));
+        ui.text_wrapped("Click this to get the sheet ready for use as a tileset. \
+                                Warning: This will modify the texture and replace any existing slices on this sheet!");
 
         ui.spacing();
         ui.spacing();
 
-        if ui.button(im_str!("Create Slices Only"), [0.0, 0.0])
+        if ui.button("Create Slices Only")
         {
-            self.create_tileset_slices(sheet, &self.tileset_slice_info(sheet.width(), sheet.height()));
+            let slices = crate::import::sheet::create_tileset_slices(&self.tilegen,
+                &crate::import::sheet::tileset_slice_info(sheet.width(), sheet.height(), &self.tilegen));
+
+            if slices.len() > 0
+            {
+                self.queue_action(Box::new(SheetAction::RemoveSlices(0, sheet.slice_count())));
+                self.queue_action(Box::new(SheetAction::AddSlices(0, slices)));
+            }
         }
 
-        ui.text_wrapped(im_str!("Only use this if you know what you are doing."));
+        ui.text_wrapped("Only use this if you know what you are doing.");
     }
 
-    fn texture_sheet_tab(&mut self, ui : &mut Ui, sheet : &mut Sheet)
+    fn texture_sheet_tab(&mut self, ui : &imgui::Ui, sheet : &mut Sheet)
     {
         let style_var_outer = ui.push_style_var(imgui::StyleVar::WindowPadding([0.0, 0.0]));
 
@@ -838,7 +606,7 @@ impl SheetDocument
             {
                 let pos = ui.cursor_screen_pos();
 
-                ui.invisible_button(im_str!("texture_drag"), [width, height]);
+                ui.invisible_button("texture_drag", [width, height]);
 
                 let hovering = ui.is_item_hovered();
 
@@ -902,7 +670,7 @@ impl SheetDocument
                     self.dragging = false;
                 }
 
-                ui.same_line(1.0);
+                ui.same_line_with_pos(1.0);
                 imgui::Image::new(imgui::TextureId::from(self.data_id), [width, height]).build(ui);
 
                 if !self.on_tilegen
@@ -954,14 +722,14 @@ impl SheetDocument
                 ui.set_scroll_y(ui.scroll_y() - delta[1]);
             }
 
-            style_color.pop(ui);
-            style_var.pop(ui);
+            style_color.pop();
+            style_var.pop();
         });
 
-        style_var_outer.pop(ui);
+        style_var_outer.pop();
     }
 
-    fn texture_slice_tab(&mut self, ui : &mut Ui, sheet : &mut Sheet)
+    fn texture_slice_tab(&mut self, ui : &imgui::Ui, sheet : &mut Sheet)
     {
         let style_var = ui.push_style_var(imgui::StyleVar::WindowPadding([1.0, 1.0]));
         let style_color = ui.push_style_color(imgui::StyleColor::ChildBg, [self.transparent_color[0], self.transparent_color[1], self.transparent_color[2], 1.0]);
@@ -1001,12 +769,12 @@ impl SheetDocument
             }
             else
             {
-                ui.text(im_str!("No slice selected!"));
+                ui.text("No slice selected!");
             }
         });
 
-        style_color.pop(ui);
-        style_var.pop(ui);
+        style_color.pop();
+        style_var.pop();
     }
 }
 
@@ -1037,14 +805,17 @@ impl Document for SheetDocument
             mouse_drawing_texture : false,
             draw_points : Vec::new(),
             dragging : false,
-            tilegen_w : 32,
-            tilegen_h : 32,
-            tilegen_x : 0,
-            tilegen_y : 0,
-            tilegen_space_x : 0,
-            tilegen_space_y : 0,
-            tilegen_rows : 0,
-            tilegen_columns : 0,
+            tilegen : TilesetGenInfo
+            {
+                width : 32,
+                height : 32,
+                x : 0,
+                y : 0,
+                space_x : 0,
+                space_y : 0,
+                rows : 0,
+                columns : 0
+            },
             tilegen_preview : true,
             on_tilegen : false
         }
@@ -1172,7 +943,8 @@ impl Document for SheetDocument
         false
     }
 
-    fn do_ui(&mut self, _packages : &mut Packages, _config : &mut EditorConfig, game: &mut GameControl, ui : &mut imgui::Ui) -> DocumentState
+    fn do_ui(&mut self, _packages : &mut Packages, _config : &mut EditorConfig,
+             _project : &mut ProjectConfig, game : &mut GameControl, ui : &imgui::Ui) -> DocumentState
     {
         self.tics = self.tics.wrapping_add(1);
 
@@ -1180,23 +952,23 @@ impl Document for SheetDocument
 
         ui.menu_bar(||
         {
-            ui.menu(im_str!("View"), true, ||
+            ui.menu("View", ||
             {
-                if imgui::MenuItem::new(im_str!("Show Origin")).selected(self.show_origin).build(ui)
+                if imgui::MenuItem::new("Show Origin").selected(self.show_origin).build(ui)
                 {
                     self.show_origin = !self.show_origin;
                 }
 
-                ui.menu(im_str!("Background Color"), true, ||
+                ui.menu("Background Color", ||
                 {
-                    imgui::ColorPicker::new(im_str!("##background_color_picker"), imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
+                    imgui::ColorPicker::new("##background_color_picker", imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
                 });
             });
-            ui.menu(im_str!("Zoom"), true, ||
+            ui.menu("Zoom", ||
             {
                 for level in &[0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0]
                 {
-                    if imgui::MenuItem::new(&im_str!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
+                    if imgui::MenuItem::new(format!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
                     {
                         self.zoom_level = *level;
                     }
@@ -1210,58 +982,58 @@ impl Document for SheetDocument
 
         if let Some(sheet) = sheet_store.get_mut(self.data_id)
         {
-            ui.columns(2, im_str!("##sheet_columns"), false);
+            ui.columns(2, "##sheet_columns", false);
 
             if let Some(sheet_controls) = imgui::ChildWindow::new("sheet_controls").size([240.0, 0.0]).draw_background(true).border(false).begin(ui)
             {
-                if let Some(tab_bar) = imgui::TabBar::new(im_str!("controls_tabbar"))
-                    .flags(TabBarFlags::FITTING_POLICY_SCROLL).begin(ui)
+                if let Some(tab_bar) = imgui::TabBar::new("controls_tabbar")
+                    .flags(imgui::TabBarFlags::FITTING_POLICY_SCROLL).begin(ui)
                 {
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Sheet")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Sheet").begin(ui)
                     {
                         self.sheet_tab(ui, sheet);
-                        tab.end(ui);
+                        tab.end();
                     }
 
                     self.on_tilegen = false;
 
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Tileset Gen")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Tileset Gen").begin(ui)
                     {
                         self.on_tilegen = true;
 
                         self.tilegen_tab(ui, sheet);
-                        tab.end(ui);
+                        tab.end();
                     }
 
-                    tab_bar.end(ui);
+                    tab_bar.end();
                 }
 
-                sheet_controls.end(ui);
+                sheet_controls.end();
             }
 
             ui.set_current_column_width(ui.item_rect_size()[0] + 10.0);
             ui.next_column();
 
-            if let Some(texture_tab_bar) = imgui::TabBar::new(im_str!("texture_tab_bar")).begin(ui)
+            if let Some(texture_tab_bar) = imgui::TabBar::new("texture_tab_bar").begin(ui)
             {
-                if let Some(tab) = imgui::TabItem::new(im_str!("Sheet View")).begin(ui)
+                if let Some(tab) = imgui::TabItem::new("Sheet View").begin(ui)
                 {
                     self.texture_sheet_tab(ui, sheet);
 
-                    tab.end(ui);
+                    tab.end();
                 }
 
                 if !self.on_tilegen
                 {
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Slice View")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Slice View").begin(ui)
                     {
                         self.texture_slice_tab(ui, sheet);
 
-                        tab.end(ui);
+                        tab.end();
                     }
                 }
 
-                texture_tab_bar.end(ui);
+                texture_tab_bar.end();
             }
         }
 
