@@ -1,5 +1,5 @@
 use crate::document::{Document, DocumentState, ActionHistory};
-use crate::editorgui::{Packages, ResourceType, EditorConfig};
+use crate::editorgui::{Packages, ResourceType, EditorConfig, ProjectConfig};
 
 use std::collections::HashSet;
 use std::any::Any;
@@ -8,16 +8,15 @@ use keeshond::gameloop::GameControl;
 use keeshond::renderer::Sheet;
 
 use keeshond_datapack::{DataId, DataHandle};
-use keeshond_treats::tilemap::{Tileset, TilesetTile, TileCollision, TileBoundingBoxData};
+use keeshond_treats::tilemap::{Tileset, TilesetTile, TileCollision, TileBoundingBoxData, TileHeightmapData};
 use keeshond_treats::visual::{SliceIndexHandle};
 
-use imgui::{im_str, Ui, TabBarFlags, StyleColor};
-
+const MAX_HEIGHTMAP_POINTS : usize = 256;
 
 enum TilesetAction
 {
     SetSheet(String, String),
-    SetTileSize(f32, f32),
+    SetTileSize(f64, f64),
     AddTile(usize, TilesetTile),
     AddTiles(usize, Vec<TilesetTile>),
     SetTile(usize, TilesetTile),
@@ -109,40 +108,107 @@ impl TilesetDocument
         false
     }
 
+    fn graph_points(&self, x1 : f64, x2 : f64, y : &Vec<f64>) -> Vec<(f64, f64)>
+    {
+        let mut points = Vec::with_capacity(y.len() + 1);
+
+        if y.len() == 1
+        {
+            points.push((x1, y[0]));
+            points.push((x2, y[0]));
+        }
+        else
+        {
+            let xstep = (x2 - x1) / ((y.len() - 1) as f64);
+
+            for i in 0..y.len()
+            {
+                points.push((x1 + xstep * i as f64, y[i]));
+            }
+        }
+
+        return points;
+    }
+
     fn draw_collision_shape(&mut self, pos : [f32; 2], size : [f32; 2], zoom : f32, draw_list : &imgui::DrawListMut, tile : &TilesetTile)
     {
         draw_list.add_rect([pos[0], pos[1]], [pos[0] + size[0], pos[1] + size[1]], imgui::ImColor32::from([0.0, 0.0, 0.0, 0.25])).filled(true).build();
 
+        let adjust_x = |x| { (pos[0] + x as f32 * zoom).max(pos[0]).min(pos[0] + size[0]).round() };
+        let adjust_y = |y| { (pos[1] + y as f32 * zoom).max(pos[1]).min(pos[1] + size[1]).round() };
+        let adjust_x_poly = |x| { (pos[0] + x as f32 * zoom).max(pos[0]).min(pos[0] + size[0] - 1.0) };
+        let adjust_y_poly = |y| { (pos[1] + y as f32 * zoom).max(pos[1]).min(pos[1] + size[1] - 1.0) };
+
         match tile.collision()
         {
             TileCollision::None => {}
             TileCollision::BoundingBox(bounding_box) =>
             {
-                let x1 = (pos[0] + bounding_box.x1 * zoom).max(pos[0]).min(pos[0] + size[0]);
-                let y1 = (pos[1] + bounding_box.y1 * zoom).max(pos[1]).min(pos[1] + size[1]);
-                let x2 = (pos[0] + bounding_box.x2 * zoom).max(pos[0]).min(pos[0] + size[0]);
-                let y2 = (pos[1] + bounding_box.y2 * zoom).max(pos[1]).min(pos[1] + size[1]);
+                let x1 = adjust_x(bounding_box.x1);
+                let y1 = adjust_y(bounding_box.y1);
+                let x2 = adjust_x(bounding_box.x2);
+                let y2 = adjust_y(bounding_box.y2);
 
-                draw_list.add_rect([x1, y1], [x2, y2], imgui::ImColor32::from([1.0, 1.0, 1.0, 0.25])).filled(true).build();
-                draw_list.add_rect([x1, y1], [x2, y2], imgui::ImColor32::from([1.0, 1.0, 1.0, 1.0])).thickness(1.0).build();
+                draw_list.add_rect([x1, y1], [x2, y2], [1.0, 1.0, 1.0, 0.25]).filled(true).build();
+                draw_list.add_rect([x1, y1], [x2, y2], [1.0, 1.0, 1.0, 1.0]).thickness(1.0).build();
+            }
+            TileCollision::Heightmap(heightmap) =>
+            {
+                if !heightmap.valid()
+                {
+                    return;
+                }
+
+                let draw_lines_xy = |line : &[(f64, f64)]|
+                {
+                    draw_list.add_line([adjust_x_poly(line[0].0), adjust_y_poly(line[0].1)],
+                                       [adjust_x_poly(line[1].0), adjust_y_poly(line[1].1)],
+                                       [1.0, 1.0, 0.0, 1.0]).thickness(1.0).build();
+                };
+                let draw_lines_yx = |line : &[(f64, f64)]|
+                {
+                    draw_list.add_line([adjust_x_poly(line[0].1), adjust_y_poly(line[0].0)],
+                                       [adjust_x_poly(line[1].1), adjust_y_poly(line[1].0)],
+                                       [1.0, 0.0, 0.0, 1.0]).thickness(1.0).build();
+                };
+
+                let (top_x1, top_x2) = heightmap.top_x_range();
+                let (bottom_x1, bottom_x2) = heightmap.bottom_x_range();
+                let (left_y1, left_y2) = heightmap.left_y_range();
+                let (right_y1, right_y2) = heightmap.right_y_range();
+
+                for line in self.graph_points(top_x1, top_x2, &heightmap.top).windows(2)
+                {
+                    draw_lines_xy(line);
+                }
+                for line in self.graph_points(bottom_x1, bottom_x2, &heightmap.bottom).windows(2)
+                {
+                    draw_lines_xy(line);
+                }
+
+                for line in self.graph_points(left_y1, left_y2, &heightmap.left).windows(2)
+                {
+                    draw_lines_yx(line);
+                }
+                for line in self.graph_points(right_y1, right_y2, &heightmap.right).windows(2)
+                {
+                    draw_lines_yx(line);
+                }
             }
         }
     }
 
-    fn slices_tab(&mut self, ui : &mut Ui, tileset : &mut Tileset, sheet_opt : Option<&Sheet>)
+    fn slices_tab(&mut self, ui : &imgui::Ui, tileset : &mut Tileset, sheet_opt : Option<&Sheet>)
     {
         let main_style = ui.clone_style();
         let source = tileset.sheet();
-        let mut package_name = imgui::ImString::with_capacity(4096);
-        let mut sheet_name = imgui::ImString::with_capacity(4096);
-
-        package_name.push_str(source.package());
-        sheet_name.push_str(source.path());
+        let mut package_name = source.package().clone();
+        let mut sheet_name = source.path().clone();
 
         let mut update_sheet = false;
 
-        update_sheet |= ui.input_text(im_str!("Package"), &mut package_name).enter_returns_true(true).build();
-        update_sheet |= ui.input_text(im_str!("Sheet"), &mut sheet_name).enter_returns_true(true).build();
+        update_sheet |= ui.input_text("Package", &mut package_name).enter_returns_true(true).build();
+        update_sheet |= ui.input_text("Sheet", &mut sheet_name).enter_returns_true(true).build();
 
         if update_sheet
         {
@@ -152,7 +218,7 @@ impl TilesetDocument
         let (width, height) = tileset.tile_size();
         let mut sizes = [width, height];
 
-        if imgui::Drag::new(im_str!("Tile Size")).display_format(im_str!("%.4f")).build_array(ui, &mut sizes)
+        if imgui::Drag::new("Tile Size").display_format("%.4f").build_array(ui, &mut sizes)
         {
             self.queue_action(Box::new(TilesetAction::SetTileSize(sizes[0], sizes[1])));
         }
@@ -161,13 +227,13 @@ impl TilesetDocument
         {
             if let Some(slice) = sheet.slice(0)
             {
-                if width != slice.texture_w || height != slice.texture_h
+                if width as f32 != slice.texture_w || height as f32 != slice.texture_h
                 {
-                    ui.same_line(0.0);
+                    ui.same_line();
 
-                    if ui.button(im_str!("!###tile_size_auto"), [0.0, 0.0])
+                    if ui.button("!###tile_size_auto")
                     {
-                        self.queue_action(Box::new(TilesetAction::SetTileSize(slice.texture_w, slice.texture_h)));
+                        self.queue_action(Box::new(TilesetAction::SetTileSize(slice.texture_w as f64, slice.texture_h as f64)));
                     }
                 }
             }
@@ -176,7 +242,7 @@ impl TilesetDocument
         ui.spacing();
         ui.separator();
         ui.spacing();
-        ui.text(&im_str!("Editing Tile {}", self.current_tile));
+        ui.text(format!("Editing Tile {}", self.current_tile));
 
         ui.spacing();
 
@@ -187,14 +253,16 @@ impl TilesetDocument
                 let index_opt = tileset.updated_slice_id(self.current_tile, sheet);
                 let mut has_slice = index_opt.is_some();
 
-                if ui.checkbox(im_str!("Has Slice"), &mut has_slice)
+                if ui.checkbox("Has Slice", &mut has_slice)
                 {
                     let mut tile = tileset.tile(self.current_tile).expect("Current tile vanished!").clone();
 
                     if has_slice
                     {
                         tile.set_slice_handle(Some(SliceIndexHandle::with_index(0)));
-                    } else {
+                    }
+                    else
+                    {
                         tile.set_slice_handle(None);
                     }
 
@@ -204,15 +272,10 @@ impl TilesetDocument
                 if let Some(index) = index_opt
                 {
                     let mut index_slider = index as u32;
-                    let mut slice_name = imgui::ImString::with_capacity(4096);
-
-                    if let Some(name) = tileset.slice_name(self.current_tile)
-                    {
-                        slice_name.push_str(name);
-                    }
+                    let mut slice_name = tileset.slice_name(self.current_tile).cloned().unwrap_or_default();
 
                     let width = ui.push_item_width(48.0);
-                    if imgui::Drag::new(im_str!("Index###slice_index")).build(ui, &mut index_slider)
+                    if imgui::Drag::new("Index###slice_index").build(ui, &mut index_slider)
                     {
                         let mut tile = tileset.tile(self.current_tile).expect("Current tile vanished!").clone();
                         tile.set_slice_handle(Some(SliceIndexHandle::with_index(index_slider as usize)));
@@ -221,13 +284,13 @@ impl TilesetDocument
                     }
                     width.pop(ui);
 
-                    ui.same_line(0.0);
+                    ui.same_line();
 
                     let width = ui.push_item_width(128.0);
-                    if ui.input_text(im_str!("Name###slice_name"), &mut slice_name).build()
+                    if ui.input_text("Name###slice_name", &mut slice_name).build()
                     {
                         let mut tile = tileset.tile(self.current_tile).expect("Current tile vanished!").clone();
-                        tile.set_slice_handle(Some(SliceIndexHandle::with_name(slice_name.to_str())));
+                        tile.set_slice_handle(Some(SliceIndexHandle::with_name(&slice_name)));
 
                         self.queue_action(Box::new(TilesetAction::SetTile(self.current_tile, tile)));
                     }
@@ -235,11 +298,11 @@ impl TilesetDocument
                 }
                 else
                 {
-                    ui.invisible_button(im_str!("###slice_placeholder"), [100.0, ui.current_font_size() + main_style.frame_padding[1] * 2.0]);
+                    ui.invisible_button("###slice_placeholder", [100.0, ui.current_font_size() + main_style.frame_padding[1] * 2.0]);
                 }
             }
 
-            if ui.button(im_str!("Create Tiles From Unused Slices"), [0.0, 0.0])
+            if ui.button("Create Tiles From Unused Slices")
             {
                 let mut tiles = Vec::new();
                 let mut used = HashSet::new();
@@ -268,15 +331,15 @@ impl TilesetDocument
         }
     }
 
-    fn collision_tab(&mut self, ui : &mut Ui, tileset : &mut Tileset)
+    fn collision_tab(&mut self, ui : &imgui::Ui, tileset : &mut Tileset)
     {
-        ui.text(&im_str!("Editing Tile {}", self.current_tile));
+        ui.text(format!("Editing Tile {}", self.current_tile));
 
         if self.extra_selection.len() > 0
         {
-            ui.same_line(0.0);
+            ui.same_line();
 
-            if ui.small_button(&im_str!("Copy To {} Others###collision_copy", self.extra_selection.len()))
+            if ui.small_button(format!("Copy To {} Others###collision_copy", self.extra_selection.len()))
             {
                 if let Some(tile) = tileset.tile(self.current_tile)
                 {
@@ -297,7 +360,9 @@ impl TilesetDocument
 
         ui.spacing();
 
-        imgui::ChildWindow::new("tileset_collision_controls").size([0.0, 160.0]).draw_background(false).border(false).build(ui, ||
+        let remaining = ui.content_region_avail();
+
+        if let Some(token) = imgui::ChildWindow::new("tileset_collision_controls").size([0.0, remaining[1] * 0.3]).draw_background(false).border(false).begin(ui)
         {
             if let Some(tile) = tileset.tile(self.current_tile)
             {
@@ -305,10 +370,11 @@ impl TilesetDocument
                 {
                     TileCollision::None => { 0 }
                     TileCollision::BoundingBox(_) => { 1 }
+                    TileCollision::Heightmap(_) => { 2 }
                 };
 
-                if imgui::ComboBox::new(im_str!("Collision Type"))
-                    .build_simple_string(ui, &mut index, &[im_str!("None"), im_str!("Bounding Box")])
+                if ui.combo_simple_string("Collision Type",
+                    &mut index, &["None", "Bounding Box", "Heightmap"])
                 {
                     let mut new_tile = tile.clone();
                     let (width, height) = tileset.tile_size();
@@ -316,6 +382,7 @@ impl TilesetDocument
                     {
                         0 => TileCollision::None,
                         1 => TileCollision::BoundingBox(TileBoundingBoxData::from_width_height(width, height)),
+                        2 => TileCollision::Heightmap(TileHeightmapData::from_width_height(width, height)),
                         _ => panic!("Unknown collision tile index")
                     };
 
@@ -324,6 +391,8 @@ impl TilesetDocument
                     self.queue_action(Box::new(TilesetAction::SetTile(self.current_tile, new_tile)));
                 }
 
+                let (tile_width, tile_height) = tileset.tile_size();
+
                 match tile.collision()
                 {
                     TileCollision::None => {},
@@ -334,22 +403,24 @@ impl TilesetDocument
                         let mut x_bounds = [new_box.x1, new_box.x2];
                         let mut y_bounds = [new_box.y1, new_box.y2];
 
-                        modified |= imgui::Drag::new(im_str!("X Bounds")).display_format(im_str!("%.4f")).build_array(ui, &mut x_bounds);
-                        modified |= imgui::Drag::new(im_str!("Y Bounds")).display_format(im_str!("%.4f")).build_array(ui, &mut y_bounds);
+                        modified |= imgui::Drag::new("X Bounds")
+                            .range(0.0, tile_width).display_format("%.4f").build_array(ui, &mut x_bounds);
+                        modified |= imgui::Drag::new("Y Bounds")
+                            .range(0.0, tile_height).display_format("%.4f").build_array(ui, &mut y_bounds);
 
                         ui.spacing();
                         ui.spacing();
-                        ui.text(im_str!("Solidity"));
+                        ui.text("Solidity");
 
                         ui.spacing();
 
-                        modified |= ui.checkbox(im_str!("Top###solid_top"), &mut new_box.solid_top);
-                        ui.same_line(0.0);
-                        modified |= ui.checkbox(im_str!("Bottom###solid_bottom"), &mut new_box.solid_bottom);
-                        ui.same_line(0.0);
-                        modified |= ui.checkbox(im_str!("Left###solid_left"), &mut new_box.solid_left);
-                        ui.same_line(0.0);
-                        modified |= ui.checkbox(im_str!("Right###solid_right"), &mut new_box.solid_right);
+                        modified |= ui.checkbox("Top###solid_top", &mut new_box.solid_top);
+                        ui.same_line();
+                        modified |= ui.checkbox("Bottom###solid_bottom", &mut new_box.solid_bottom);
+                        ui.same_line();
+                        modified |= ui.checkbox("Left###solid_left", &mut new_box.solid_left);
+                        ui.same_line();
+                        modified |= ui.checkbox("Right###solid_right", &mut new_box.solid_right);
 
                         if modified
                         {
@@ -362,20 +433,104 @@ impl TilesetDocument
                             self.queue_action(Box::new(TilesetAction::SetTile(self.current_tile, new_tile)));
                         }
                     }
+                    TileCollision::Heightmap(heightmap) =>
+                    {
+                        self.heightmap_controls(ui, tile, heightmap.clone(), tile_width, tile_height);
+                    }
                 }
             }
-        });
+
+            token.end();
+        }
     }
 
-    fn flags_tab(&mut self, ui : &mut Ui, tileset : &mut Tileset)
+    fn heightmap_side_list(&mut self, ui : &imgui::Ui, side : &mut Vec<f64>, side_name : &str, max_height : f64, default_height : f64) -> bool
     {
-        ui.text(&im_str!("Editing Tile {}", self.current_tile));
+        let mut modified = false;
+
+        ui.indent();
+        ui.same_line_with_pos(60.0);
+
+        let item_width = ui.push_item_width(120.0);
+        let mut num_points = side.len() as i32;
+
+        if imgui::InputInt::new(ui, format!("Points###{} Points", side_name), &mut num_points).build()
+        {
+            side.resize((num_points as usize).clamp(1, MAX_HEIGHTMAP_POINTS), side.last().cloned().unwrap_or(default_height));
+            modified = true;
+        }
+
+        let style_point_spacing = ui.push_style_var(imgui::StyleVar::ItemSpacing([0.0, 0.0]));
+        let style_point_padding = ui.push_style_var(imgui::StyleVar::FramePadding([3.0, 3.0]));
+
+        for i in 0..side.len()
+        {
+            modified |= imgui::Drag::new(format!("[{}]###{} {}", i, side_name, i))
+                .range(0.0, max_height).display_format("%.4f").build(ui, &mut side[i]);
+        }
+
+        style_point_padding.pop();
+        style_point_spacing.pop();
+
+        item_width.pop(ui);
+
+        ui.unindent();
+        ui.spacing();
+
+        modified
+    }
+
+    fn heightmap_controls(&mut self, ui : &imgui::Ui, tile : &TilesetTile, mut heightmap : TileHeightmapData, tile_width : f64, tile_height : f64)
+    {
+        let mut modified = false;
+
+        ui.align_text_to_frame_padding();
+        ui.text("Top");
+        modified |= self.heightmap_side_list(ui, &mut heightmap.top, "Top", tile_height, 0.0);
+
+        ui.align_text_to_frame_padding();
+        ui.text("Bottom");
+        modified |= self.heightmap_side_list(ui, &mut heightmap.bottom, "Bottom", tile_height, tile_height);
+
+        ui.align_text_to_frame_padding();
+        ui.text("Left");
+        modified |= self.heightmap_side_list(ui, &mut heightmap.left, "Left", tile_width, 0.0);
+
+        ui.align_text_to_frame_padding();
+        ui.text("Right");
+        modified |= self.heightmap_side_list(ui, &mut heightmap.right, "Right", tile_width, tile_width);
+
+        ui.spacing();
+        ui.spacing();
+        ui.text("Solidity");
+
+        ui.spacing();
+
+        modified |= ui.checkbox("Top###solid_top", &mut heightmap.solid_top);
+        ui.same_line();
+        modified |= ui.checkbox("Bottom###solid_bottom", &mut heightmap.solid_bottom);
+        ui.same_line();
+        modified |= ui.checkbox("Left###solid_left", &mut heightmap.solid_left);
+        ui.same_line();
+        modified |= ui.checkbox("Right###solid_right", &mut heightmap.solid_right);
+
+        if modified
+        {
+            let mut new_tile = tile.clone();
+            new_tile.set_collision(TileCollision::Heightmap(heightmap));
+            self.queue_action(Box::new(TilesetAction::SetTile(self.current_tile, new_tile)));
+        }
+    }
+
+    fn flags_tab(&mut self, ui : &imgui::Ui, tileset : &mut Tileset)
+    {
+        ui.text(format!("Editing Tile {}", self.current_tile));
 
         if self.extra_selection.len() > 0
         {
-            ui.same_line(0.0);
+            ui.same_line();
 
-            if ui.small_button(&im_str!("Copy To {} Others###flag_copy", self.extra_selection.len()))
+            if ui.small_button(format!("Copy To {} Others###flag_copy", self.extra_selection.len()))
             {
                 if let Some(tile) = tileset.tile(self.current_tile)
                 {
@@ -425,25 +580,23 @@ impl TilesetDocument
                         self.queue_action(Box::new(TilesetAction::SetTile(self.current_tile, new_tile)));
                     }
 
-                    ui.same_line(ui.window_content_region_max()[0] - 70.0);
+                    ui.same_line_with_pos(ui.window_content_region_max()[0] - 70.0);
 
-                    ui.checkbox_flags(&im_str!("Filter###filter_flag_{}", i), &mut self.filter_flags, 1 << i);
+                    ui.checkbox_flags(format!("Filter###filter_flag_{}", i), &mut self.filter_flags, 1 << i);
                 }
             }
         });
     }
 
-    fn flag_names_tab(&mut self, ui : &mut Ui, tileset : &mut Tileset)
+    fn flag_names_tab(&mut self, ui : &imgui::Ui, tileset : &mut Tileset)
     {
         for i in 0..32
         {
             if let Some(name) = tileset.flag_name(i)
             {
-                let mut flag_name = imgui::ImString::with_capacity(4096);
+                let mut flag_name = name.clone();
 
-                flag_name.push_str(name);
-
-                if ui.input_text(&im_str!("Flag {}", i), &mut flag_name).enter_returns_true(true).build()
+                if ui.input_text(format!("Flag {}", i), &mut flag_name).enter_returns_true(true).build()
                 {
                     self.queue_action(Box::new(TilesetAction::RenameFlag(i, flag_name.to_string())));
                 }
@@ -451,11 +604,11 @@ impl TilesetDocument
         }
     }
 
-    fn tile_view(&mut self, ui : &mut Ui, tileset : &mut Tileset, texture_id : usize, sheet_opt : Option<&Sheet>, collision_open : bool, flags_open : bool)
+    fn tile_view(&mut self, ui : &imgui::Ui, tileset : &mut Tileset, texture_id : usize, sheet_opt : Option<&Sheet>, collision_open : bool, flags_open : bool)
     {
         let main_style = ui.clone_style();
 
-        if ui.button(im_str!("Add"), [0.0, 0.0])
+        if ui.button("Add")
         {
             let new_tile;
 
@@ -471,17 +624,17 @@ impl TilesetDocument
             self.queue_action(Box::new(TilesetAction::AddTile(self.current_tile, new_tile)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Up"), [0.0, 0.0]) && self.current_tile > 0
+        if ui.button("Up") && self.current_tile > 0
         {
             self.queue_action(Box::new(TilesetAction::MoveTile(self.current_tile, self.current_tile - 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Down"), [0.0, 0.0])
+        if ui.button("Down")
         {
             self.queue_action(Box::new(TilesetAction::MoveTile(self.current_tile, self.current_tile + 1)));
         }
         ui.same_line_with_spacing(0.0, 4.0);
-        if ui.button(im_str!("Remove"), [0.0, 0.0])
+        if ui.button("Remove")
         {
             self.queue_action(Box::new(TilesetAction::RemoveTile(self.current_tile)));
         }
@@ -497,8 +650,8 @@ impl TilesetDocument
             let start_pos = ui.cursor_screen_pos();
 
             let (orig_tile_width, orig_tile_height) = tileset.tile_size();
-            let mut tile_width = orig_tile_width * self.zoom_level;
-            let mut tile_height = orig_tile_height * self.zoom_level;
+            let mut tile_width = orig_tile_width as f32 * self.zoom_level;
+            let mut tile_height = orig_tile_height as f32 * self.zoom_level;
             let base_width = tile_width.max(1.0);
 
             tile_width = tile_width.max(8.0).min(128.0);
@@ -518,7 +671,7 @@ impl TilesetDocument
             {
                 if i % columns > 0
                 {
-                    ui.same_line(0.0);
+                    ui.same_line();
                 }
 
                 let cursor_pos = ui.cursor_pos();
@@ -526,7 +679,7 @@ impl TilesetDocument
 
                 ui.set_cursor_pos([cursor_pos[0], cursor_pos[1]]);
 
-                if imgui::Selectable::new(&im_str!("###tile_{}", i))
+                if imgui::Selectable::new(format!("###tile_{}", i))
                     .selected(self.current_tile == i || self.extra_selection.contains(&i)).size(button_size).build(ui)
                 {
                     if ui.io().key_ctrl
@@ -558,7 +711,7 @@ impl TilesetDocument
 
                 if ui.is_item_hovered()
                 {
-                    ui.tooltip_text(&im_str!("Tile {}", i));
+                    ui.tooltip_text(format!("Tile {}", i));
                 }
             }
 
@@ -570,7 +723,7 @@ impl TilesetDocument
                 {
                     let x = start_pos[0] + (i % columns) as f32 * tile_width;
                     let y = start_pos[1] + (i / columns) as f32 * tile_height;
-                    let zoom = tile_width / orig_tile_width;
+                    let zoom = tile_width / orig_tile_width as f32;
 
                     if let Some(tile) = tileset.tile(i)
                     {
@@ -600,13 +753,13 @@ impl TilesetDocument
             let x2 = x1 + tile_width;
             let y2 = y1 + tile_height;
 
-            draw_list.add_rect([x1, y1], [x2, y2], main_style.colors[StyleColor::Button as usize]).thickness(2.0).build();
+            draw_list.add_rect([x1, y1], [x2, y2], main_style.colors[imgui::StyleColor::Button as usize]).thickness(2.0).build();
 
-            style_slice.pop(ui);
+            style_slice.pop();
         });
 
-        colors_slice_list.pop(ui);
-        style_slice_list.pop(ui);
+        colors_slice_list.pop();
+        style_slice_list.pop();
     }
 }
 
@@ -698,7 +851,8 @@ impl Document for TilesetDocument
         false
     }
 
-    fn do_ui(&mut self, packages : &mut Packages, _config : &mut EditorConfig, game : &mut GameControl, ui : &mut Ui) -> DocumentState
+    fn do_ui(&mut self, packages : &mut Packages, _config : &mut EditorConfig,
+             _project : &mut ProjectConfig, game : &mut GameControl, ui : &imgui::Ui) -> DocumentState
     {
         let mut result = DocumentState::new();
         let sheet_store = game.res().store_mut::<Sheet>();
@@ -706,23 +860,23 @@ impl Document for TilesetDocument
 
         ui.menu_bar(||
         {
-            ui.menu(im_str!("View"), true, ||
+            ui.menu("View", ||
             {
-                if imgui::MenuItem::new(im_str!("Show Collision Shapes in Tile List")).selected(self.show_colliders_in_list).build(ui)
+                if imgui::MenuItem::new("Show Collision Shapes in Tile List").selected(self.show_colliders_in_list).build(ui)
                 {
                     self.show_colliders_in_list = !self.show_colliders_in_list;
                 }
 
-                ui.menu(im_str!("Background Color"), true, ||
+                ui.menu("Background Color", ||
                 {
-                    imgui::ColorPicker::new(im_str!("##background_color_picker"), imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
+                    imgui::ColorPicker::new("##background_color_picker", imgui::EditableColor::Float3(&mut self.transparent_color)).build(ui);
                 });
             });
-            ui.menu(im_str!("Zoom"), true, ||
+            ui.menu("Zoom", ||
             {
                 for level in &[0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0]
                 {
-                    if imgui::MenuItem::new(&im_str!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
+                    if imgui::MenuItem::new(format!("{}%", (level * 100.0) as i32)).selected(self.zoom_level == *level).build(ui)
                     {
                         self.zoom_level = *level;
                     }
@@ -750,7 +904,7 @@ impl Document for TilesetDocument
                 }
             };
 
-            ui.columns(2, im_str!("##tileset_columns"), true);
+            ui.columns(2, "##tileset_columns", true);
 
             if !self.left_column_set
             {
@@ -766,42 +920,42 @@ impl Document for TilesetDocument
             {
                 self.current_tile = self.current_tile.min(tileset.tile_count().max(1) - 1);
 
-                if let Some(tab_bar) = imgui::TabBar::new(im_str!("tileset_tab_bar"))
-                    .flags(TabBarFlags::FITTING_POLICY_SCROLL).begin(ui)
+                if let Some(tab_bar) = imgui::TabBar::new("tileset_tab_bar")
+                    .flags(imgui::TabBarFlags::FITTING_POLICY_SCROLL).begin(ui)
                 {
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Slices")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Slices").begin(ui)
                     {
                         self.slices_tab(ui, tileset, sheet_opt);
 
-                        tab.end(ui);
+                        tab.end();
                     }
 
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Collision")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Collision").begin(ui)
                     {
                         collision_open = true;
                         self.collision_tab(ui, tileset);
 
-                        tab.end(ui);
+                        tab.end();
                     }
 
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Flags")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Flags").begin(ui)
                     {
                         flags_open = true;
                         self.flags_tab(ui, tileset);
 
-                        tab.end(ui);
+                        tab.end();
                     }
 
-                    if let Some(tab) = imgui::TabItem::new(im_str!("Flag Names")).begin(ui)
+                    if let Some(tab) = imgui::TabItem::new("Flag Names").begin(ui)
                     {
                         flag_names_open = true;
 
                         self.flag_names_tab(ui, tileset);
 
-                        tab.end(ui);
+                        tab.end();
                     }
 
-                    tab_bar.end(ui);
+                    tab_bar.end();
                 }
 
                 if !flag_names_open
@@ -814,19 +968,19 @@ impl Document for TilesetDocument
                     {
                         self.tile_view(ui, tileset, texture_id, sheet_opt, collision_open, flags_open);
 
-                        tile_view.end(ui);
+                        tile_view.end();
                     }
                 }
 
-                tileset_controls.end(ui);
+                tileset_controls.end();
             }
 
             ui.next_column();
 
             let remaining = ui.content_region_avail();
             let (tile_width, tile_height) = tileset.tile_size();
-            let zoom = (remaining[0] / tile_width).min(remaining[1] / tile_height).floor().max(1.0).min(16.0);
-            let size = [zoom * tile_width, zoom * tile_height];
+            let zoom = (remaining[0] / tile_width as f32).min(remaining[1] / tile_height as f32).floor().max(1.0).min(16.0);
+            let size = [zoom * tile_width as f32, zoom * tile_height as f32];
 
             let style_tile_view = ui.push_style_var(imgui::StyleVar::WindowPadding([1.0, 1.0]));
             let colors_tile_view = ui.push_style_color(imgui::StyleColor::ChildBg, [self.transparent_color[0], self.transparent_color[1], self.transparent_color[2], 1.0]);
@@ -848,8 +1002,8 @@ impl Document for TilesetDocument
                 }
             });
 
-            colors_tile_view.pop(ui);
-            style_tile_view.pop(ui);
+            colors_tile_view.pop();
+            style_tile_view.pop();
         }
 
         result
