@@ -1,17 +1,18 @@
+use image::RgbaImage;
 use crate::editorgui::{Packages, ResourceType};
 
 use keeshond::renderer::{SpriteSlice, Sheet};
 use keeshond::util::{BakedRect, Rect};
 
 use keeshond_datapack::{DataStore, DataHandle, DataId};
-use keeshond_treats::visual::{AnimationCel, Costume, DrawableItem, MultiTileSprite};
+use keeshond_treats::visual::{AnimationCel, Costume, DrawableItem, MultiTileSprite, TileSprite};
 
 use num::Float;
 use toodee::TooDeeOps;
 
-use imgui::{ImStr, im_str, FontId};
 use keeshond_treats::tilemap::{Tilemap, Tileset};
 use keeshond_treats::spawnable::Representation;
+use log::Level;
 
 pub const ICON_PACKAGE : &'static str = "a";
 pub const ICON_SHEET : &'static str = "b";
@@ -51,7 +52,12 @@ pub const ICON_LOCKED : &'static str = "H";
 
 pub const ICON_FONT_SIZE : f32 = 16.0;
 
-pub const REPRESENTATION_NAMES : [&ImStr; 3] = [im_str!("None"), im_str!("Actor"), im_str!("Tilemap")];
+pub const REPRESENTATION_NAMES : [&'static str; 3] = ["None", "Actor", "Tilemap"];
+
+const PASCAL_DISALLOWED_CHARS : [char; 33] = [' ', '!', '"', '#', '$', '%', '&', '\'', '(', ')',
+    '*', '+', ',', '-', '.', '/', ':', ';', '<', '=',
+    '>', '?', '@', '[', '\\', ']', '^', '_', '`', '{',
+    '|', '}', '~'];
 
 pub fn representation_icon(representation : &Representation) -> &'static str
 {
@@ -63,9 +69,9 @@ pub fn representation_icon(representation : &Representation) -> &'static str
     }
 }
 
-pub fn window_placeholder_text(text : &ImStr, ui : &imgui::Ui)
+pub fn window_placeholder_text(text : &str, ui : &imgui::Ui)
 {
-    let size = ui.calc_text_size(text, true, 0.0);
+    let size = ui.calc_text_size(text);
     let avail = ui.content_region_avail();
     let mut cursor_pos = ui.cursor_pos();
     cursor_pos[0] += avail[0] / 2.0 - size[0] / 2.0;
@@ -74,9 +80,9 @@ pub fn window_placeholder_text(text : &ImStr, ui : &imgui::Ui)
     ui.text_disabled(text);
 }
 
-pub fn right_aligned_label(text : &imgui::ImStr, ui : &mut imgui::Ui)
+pub fn right_aligned_label(text : &str, frame_padding : bool, ui : &imgui::Ui)
 {
-    let size = ui.calc_text_size(text, true, 0.0);
+    let size = ui.calc_text_size(text);
     let style = ui.clone_style();
 
     let cursor_pos = ui.cursor_pos();
@@ -89,7 +95,12 @@ pub fn right_aligned_label(text : &imgui::ImStr, ui : &mut imgui::Ui)
     }
 
     ui.set_cursor_pos([cursor_pos[0] + advance, cursor_pos[1]]);
-    ui.align_text_to_frame_padding();
+
+    if frame_padding
+    {
+        ui.align_text_to_frame_padding();
+    }
+
     ui.text(text);
 }
 
@@ -103,6 +114,16 @@ pub fn underline_current_item(drop : f32, extend : f32, thickness : f32, ui : &i
                        [size_max[0] + extend, size_max[1] + drop], color).thickness(thickness).build();
 }
 
+pub fn column_separator(ui : &imgui::Ui)
+{
+    let draw_list = ui.get_window_draw_list();
+    let color = ui.style_color(imgui::StyleColor::Separator);
+    let pos = ui.cursor_screen_pos();
+    let width = ui.window_content_region_width();
+    draw_list.add_line([pos[0], pos[1]],
+                       [pos[0] + width, pos[1]], color).thickness(1.0).build();
+}
+
 pub fn error_text_color(ui : &imgui::Ui) -> [f32; 4]
 {
     let mut color = ui.style_color(imgui::StyleColor::Text);
@@ -115,6 +136,49 @@ pub fn error_text_color(ui : &imgui::Ui) -> [f32; 4]
     color
 }
 
+pub fn text_color_for_log_level(level : log::Level, ui : &imgui::Ui) -> [f32; 4]
+{
+    let mut color = ui.style_color(imgui::StyleColor::Text);
+
+    match level
+    {
+        Level::Error =>
+        {
+            color[0] = (color[0] + 2.0) / 3.0;
+            color[1] /= 3.0;
+            color[2] /= 3.0;
+        }
+        Level::Warn =>
+        {
+            color[0] = (color[0] + 2.0) / 3.0;
+            color[1] = (color[1] + 1.0) / 3.0;
+            color[2] /= 3.0;
+        }
+        Level::Info =>
+        {
+            color[0] /= 3.0;
+            color[1] = (color[1] + 1.0) / 3.0;
+            color[2] = (color[2] + 2.0) / 3.0;
+        }
+        Level::Debug =>
+        {
+            color[0] = (color[0] + 1.5) / 3.0;
+            color[1] /= 3.0;
+            color[2] = (color[2] + 2.0) / 3.0;
+        }
+        Level::Trace =>
+        {
+            color[0] /= 3.0;
+            color[1] = (color[1] + 2.0) / 3.0;
+            color[2] /= 3.0;
+        }
+    }
+
+    color[3] = 1.0;
+
+    color
+}
+
 pub struct BoxFitResult<T : Float>
 {
     pub x : T,
@@ -200,36 +264,79 @@ pub fn double_bool_to_option_bool(is_some : bool, inner_value : bool) -> Option<
     Some(inner_value)
 }
 
-pub fn get_icon_font(ui : &imgui::Ui) -> FontId
+pub fn get_icon_font(ui : &imgui::Ui) -> imgui::FontId
 {
     ui.fonts().fonts()[ui.fonts().fonts().len() - 1]
 }
 
-pub fn icon_label(icon : &str, name : &str, ui : &mut imgui::Ui)
+pub fn icon_label(icon : &str, name : &str, ui : &imgui::Ui)
 {
     let old_cursor = ui.cursor_pos();
     let height = ui.current_font().font_size;
     let icon_font = get_icon_font(ui);
 
     let font = ui.push_font(icon_font);
-    let icon_size = ui.calc_text_size(&im_str!("{}", icon), true, 0.0);
+    let icon_size = ui.calc_text_size(icon);
     ui.set_cursor_pos([old_cursor[0] + ((ICON_FONT_SIZE - icon_size[0]) / 2.0), old_cursor[1]]);
 
-    ui.text(&im_str!("{}", icon));
-    font.pop(ui);
+    ui.text(icon);
+    font.pop();
 
-    ui.same_line(0.0);
+    ui.same_line();
     ui.set_cursor_pos([old_cursor[0] + 22.0, old_cursor[1] + ((icon_size[1] - height) / 2.0)]);
-    ui.text(&im_str!("{}", name));
+    ui.text(name);
 }
 
-pub fn height_with_icon(ui : &mut imgui::Ui) -> f32
+pub fn height_with_icon(ui : &imgui::Ui) -> f32
 {
-    let size = ui.calc_text_size(im_str!(""), true, 0.0);
+    let size = ui.calc_text_size("");
 
     size[1].max(15.0)
 }
 
+pub fn data_path_edit(ui : &imgui::Ui, label : &str, package : &mut String, name : &mut String) -> bool
+{
+    let mut changed = false;
+
+    let width = ui.push_item_width(120.0);
+    changed |= ui.input_text(format!("/###{} Package", label), package).build();
+    width.pop(ui);
+
+    ui.same_line_with_spacing(0.0, 4.0);
+
+    let width = ui.push_item_width(160.0);
+    changed |= ui.input_text(format!("{} Package/Name", label), name).build();
+    width.pop(ui);
+
+    changed
+}
+
+pub fn name_to_pascal_case(name : &str) -> String
+{
+    let parts : Vec<String> = name.split(&PASCAL_DISALLOWED_CHARS[..]).map(|part| part.to_string()).collect();
+    let mut new_name = String::new();
+
+    for part in &parts
+    {
+        let mut start_of_word = true;
+
+        for letter in part.chars()
+        {
+            if start_of_word
+            {
+                new_name.push(letter.to_ascii_uppercase());
+                start_of_word = false;
+            }
+            else
+            {
+                new_name.push(letter.to_ascii_lowercase());
+            }
+        }
+    }
+
+    new_name
+}
+
 pub fn tile_size(tilemap : &Tilemap, tileset_store : &DataStore<Tileset>) -> (f64, f64)
 {
     let mut tile_w = 16.0;
@@ -403,6 +510,56 @@ pub fn tileset_image_from_slice(tileset : &mut Tileset, texture_id : usize, shee
     return imgui::Image::new(imgui::TextureId::from(0), image_size);
 }
 
+pub fn tile_drawable_item(tileset : &DataHandle<Tileset>, tile_index : u32, tileset_store : &mut DataStore<Tileset>,
+                             sheet_store : &mut DataStore<Sheet>, packages : &mut Packages) -> DrawableItem
+{
+    if tileset_store.package_loaded(tileset.package())
+    {
+        if let Some(tileset) = tileset_store.get_mut(tileset.resolve_if_loaded(tileset_store))
+        {
+            if sheet_store.package_loaded(tileset.sheet().package())
+            {
+                if let Ok(sheet_id) = sheet_store.get_id(tileset.sheet().package(), tileset.sheet().path())
+                {
+                    if let Some(sheet) = sheet_store.get(sheet_id)
+                    {
+                        if let Some(tile) = Tilemap::drawable_positioned_tile(tileset, sheet, 0.0, 0.0, tile_index)
+                        {
+                            let (tile_w, tile_h) = tileset.tile_size();
+
+                            return DrawableItem::TileSprite(TileSprite
+                            {
+                                texture_x : tile.texture_x,
+                                texture_y : tile.texture_y,
+                                texture_w : tile.texture_w,
+                                texture_h : tile.texture_h,
+                                tile_w : tile_w as f32,
+                                tile_h : tile_h as f32,
+                                sheet_id,
+                                shader_id : 0,
+                                r : 1.0,
+                                g : 1.0,
+                                b : 1.0,
+                                alpha : 1.0
+                            });
+                        }
+                    }
+                }
+            }
+            else
+            {
+                packages.load_package_later(ResourceType::Sheet, tileset.sheet().package());
+            }
+        }
+    }
+    else
+    {
+        packages.load_package_later(ResourceType::Tileset, tileset.package());
+    }
+
+    return DrawableItem::None;
+}
+
 pub fn tilemap_drawable_item(tilemap : &Tilemap, rect : BakedRect<f64>, tileset_store : &mut DataStore<Tileset>,
                         sheet_store : &mut DataStore<Sheet>, packages : &mut Packages) -> DrawableItem
 {
@@ -445,7 +602,6 @@ struct FloodFillInfo
     min_x : usize,
     max_x : usize,
     y : usize,
-    down : bool,
     extend_left : bool,
     extend_right : bool
 }
@@ -465,12 +621,11 @@ pub fn flood_fill<A, T, S>(target : &mut toodee::TooDee<A>, x : usize, y : usize
 
     ranges.push(FloodFillInfo
     {
-        min_x: x,
-        max_x: x,
+        min_x : x,
+        max_x : x,
         y,
-        down: false,
-        extend_left: true,
-        extend_right: true
+        extend_left : true,
+        extend_right : true
     });
 
     while !ranges.is_empty()
@@ -513,73 +668,170 @@ pub fn flood_fill<A, T, S>(target : &mut toodee::TooDee<A>, x : usize, y : usize
             if extended_max_x < width - 1 { extended_max_x += 1; }
         }
 
-        let mut add_range = |new_y, down|
+        let mut add_range = |new_y|
+        {
+            let mut range_active = false;
+            let mut range_count : usize = 0;
+            let mut range_start : usize = 0;
+
+            for x in extended_min_x..extended_max_x + 1
             {
-                let mut range_active = false;
-                let mut range_count : usize = 0;
-                let mut range_start : usize = 0;
-
-                for x in extended_min_x..extended_max_x + 1
+                if test_pixel(target, x, new_y)
                 {
-                    if test_pixel(target, x, new_y)
+                    if !range_active
                     {
-                        if !range_active
+                        range_start = x;
+                        range_active = true;
+                    }
+                }
+                else
+                {
+                    if range_active
+                    {
+                        ranges.push(FloodFillInfo
                         {
-                            range_start = x;
-                            range_active = true;
-                        }
-                    }
-                    else
-                    {
-                        if range_active
-                        {
-                            ranges.push(FloodFillInfo
-                            {
-                                min_x: range_start,
-                                max_x: x - 1,
-                                y: new_y,
-                                down,
-                                extend_left: range_count == 0,
-                                extend_right: false
-                            });
+                            min_x : range_start,
+                            max_x : x - 1,
+                            y : new_y,
+                            extend_left : range_count == 0,
+                            extend_right : false
+                        });
 
-                            range_active = false;
-                            range_count += 1;
-                        }
+                        range_active = false;
+                        range_count += 1;
                     }
                 }
+            }
 
-                if range_active
+            if range_active
+            {
+                ranges.push(FloodFillInfo
                 {
-                    ranges.push(FloodFillInfo
-                    {
-                        min_x: range_start,
-                        max_x: extended_max_x,
-                        y : new_y,
-                        down,
-                        extend_left: range_count == 0,
-                        extend_right: false
-                    });
+                    min_x : range_start,
+                    max_x : extended_max_x,
+                    y : new_y,
+                    extend_left : range_count == 0,
+                    extend_right : false
+                });
 
-                    range_count += 1;
-                }
+                range_count += 1;
+            }
 
-                if range_count > 0
+            if range_count > 0
+            {
+                if let Some(range) = ranges.last_mut()
                 {
-                    if let Some(range) = ranges.last_mut()
-                    {
-                        range.extend_right = true;
-                    }
+                    range.extend_right = true;
                 }
-            };
+            }
+        };
 
         if range.y > 0
         {
-            add_range(range.y - 1, false);
+            add_range(range.y - 1);
         }
         if range.y < height - 1
         {
-            add_range(range.y + 1, true);
+            add_range(range.y + 1);
+        }
+    }
+}
+
+struct TransparentPixel
+{
+    x : u32, y : u32, set : bool
+}
+
+pub fn defringe_image(texture : &mut RgbaImage, wrap : bool)
+{
+    let mut trans_pixels = Vec::new();
+    let mut processed_trans = vec![false; (texture.width() * texture.height()) as usize];
+    let mut processed_trans_buffer = vec![false; (texture.width() * texture.height()) as usize];
+
+    for y in 0..texture.height()
+    {
+        for x in 0..texture.width()
+        {
+            let pixel = texture.get_pixel_mut(x, y);
+
+            if pixel[3] == 0
+            {
+                trans_pixels.push(TransparentPixel
+                {
+                    x,
+                    y,
+                    set: false
+                })
+            }
+        }
+    }
+
+    let mut colors = Vec::new();
+
+    while trans_pixels.len() > 0
+    {
+        let mut processed = false;
+
+        for entry in &mut trans_pixels
+        {
+            colors.clear();
+            let (x, y) = (entry.x, entry.y);
+
+            for (offset_x, offset_y) in &[(0, -1), (-1, 0), (1, 0), (0, 1)]
+            {
+                let check_x = x as i32 + offset_x;
+                let check_y = y as i32 + offset_y;
+
+                if wrap || (check_x >= 0 && check_x < texture.width() as i32
+                    && check_y >= 0 && check_y < texture.height() as i32)
+                {
+                    let check_x_wrapped = check_x as u32 % texture.width();
+                    let check_y_wrapped = check_y as u32 % texture.height();
+                    let other_pixel = texture.get_pixel(check_x_wrapped, check_y_wrapped);
+
+                    if other_pixel[3] > 0 || processed_trans[(check_y_wrapped * texture.width() + check_x_wrapped) as usize]
+                    {
+                        colors.push((other_pixel[0], other_pixel[1], other_pixel[2]));
+                    }
+                }
+            }
+
+            if colors.len() > 0
+            {
+                let mut red = 0.0;
+                let mut green = 0.0;
+                let mut blue = 0.0;
+
+                for (other_r, other_g, other_b) in &colors
+                {
+                    red += *other_r as f32;
+                    green += *other_g as f32;
+                    blue += *other_b as f32;
+                }
+
+                red /= colors.len() as f32;
+                green /= colors.len() as f32;
+                blue /= colors.len() as f32;
+
+                let pixel = texture.get_pixel_mut(x, y);
+
+                pixel[0] = red as u8;
+                pixel[1] = green as u8;
+                pixel[2] = blue as u8;
+
+                processed = true;
+                entry.set = true;
+                processed_trans_buffer[(y * texture.width() + x) as usize] = true;
+            }
+        }
+
+        trans_pixels.retain(|entry| entry.set == false);
+        processed_trans.copy_from_slice(&processed_trans_buffer[..]);
+
+        // Don't get stuck if there are no opaque pixels
+        if !processed
+        {
+            break;
         }
     }
 }
